<?php

namespace App\Http\Controllers;

use App\Models\Almacen;
use Illuminate\Http\Request;
use App\Http\Requests\StoreAlmacenRequest;
use App\Http\Requests\UpdateAlmacenRequest;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Validator;

class AlmacenController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        return view('almacen.index');
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(StoreAlmacenRequest $request)
    {
        // Validación
        $validator = Validator::make(
            $request->all(),
            [
                'nombre' => 'required|string|max:50|unique:almacenes,nombre_almacen,NULL,id,sucursal_id,' . $request->sucursal_id,
                'sucursal_id' => 'required|exists:sucursales,id',
                'descripcion' => 'required|string|max:255',
                'direccion' => 'required|string|max:255',
            ],
            [
                'nombre.required' => 'El nombre del almacén es obligatorio',
                'nombre.unique' => 'El nombre del almacén ya está registrado para esta sucursal',
                'sucursal_id.required' => 'La sucursal es obligatoria',
                'sucursal_id.exists' => 'La sucursal seleccionada no es válida',
                'descripcion.required' => 'La descripción es obligatoria',
                'direccion.required' => 'La dirección es obligatoria',
            ]
        );

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()], 422);
        }

        try {
            // Crear el almacén
            $almacen = Almacen::create([
                'nombre_almacen' => $request->nombre,
                'sucursal_id' => $request->sucursal_id,
                'descripcion' => $request->descripcion,
                'ubicacion' => $request->direccion,
            ]);

            return response()->json(['mensaje' => 'Almacén creado exitosamente', 'almacen' => $almacen], 201);
        } catch (\Exception $e) {
            // Loguear el error para depuración
            Log::error('Error al registrar el almacén: ' . $e->getMessage());

            // Retornar respuesta de error genérico
            return response()->json(['mensaje' => 'Ocurrió un error al registrar el almacén.'], 500);
        }
    }

    /**
     * Display the specified resource.
     */
    public function show(Almacen $almacen)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(Request $request)
    {
        $idAlmacen = $request->input('idRegistroParaEditar');
        $dataAlmacen = Almacen::findOrFail($idAlmacen);
        return response()->json($dataAlmacen);
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(UpdateAlmacenRequest $request)
    {
        // Validación
        $idAlmacen = $request->input('idAlmacenEdicion');
        $validator = Validator::make(
            $request->all(),
            [
                'nombre' => 'required|string|max:50|unique:almacenes,nombre_almacen,' . $idAlmacen . ',id,sucursal_id,' . $request->sucursal_id,
                'sucursal_id' => 'required|exists:sucursales,id',
                'descripcion' => 'required|string|max:255',
                'direccion' => 'required|string|max:255',
            ],
            [
                'nombre.required' => 'El nombre del almacén es obligatorio',
                'nombre.unique' => 'El nombre del almacén ya está registrado para esta sucursal',
                'sucursal_id.required' => 'La sucursal es obligatoria',
                'sucursal_id.exists' => 'La sucursal seleccionada no es válida',
                'descripcion.required' => 'La descripción es obligatoria',
                'direccion.required' => 'La dirección es obligatoria',
            ]
        );

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()], 422);
        }

        try {
            // Encontrar el almacén existente
            $almacen = Almacen::findOrFail($idAlmacen);

            // Actualizar el almacén
            $almacen->update([
                'nombre_almacen' => $request->nombre,
                'sucursal_id' => $request->sucursal_id,
                'descripcion' => $request->descripcion,
                'ubicacion' => $request->direccion,
            ]);

            return response()->json(['mensaje' => 'Almacén actualizado exitosamente', 'almacen' => $almacen], 200);
        } catch (\Exception $e) {
            // Loguear el error para depuración
            Log::error('Error al actualizar el almacén: ' . $e->getMessage());

            // Retornar respuesta de error genérico
            return response()->json(['mensaje' => 'Ocurrió un error al actualizar el almacén.'], 500);
        }
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Almacen $almacen)
    {
        //
    }

    public function obtenerAlmacenes()
    {
        $routeName = 'almacen'; // Puedes modificar esto según el módulo
        $almacenes = Almacen::withCount('productos')->get()->map(function ($datosDelObjeto) use ($routeName) {
            return [
                'id'                => $datosDelObjeto->id,
                'nombre'            => $datosDelObjeto->nombre_almacen,
                'descripcion'       => $datosDelObjeto->descripcion,
                'estatus'           => $datosDelObjeto->estatus,
                'ubicacion'         => $datosDelObjeto->ubicacion,
                'estatus'           => $datosDelObjeto->estatus,
                'sucursal'          => $datosDelObjeto->sucursal->nombre_sucursal,
                'cantArticulos'     => $datosDelObjeto->productos_count,
                'acciones'          => view('components.accionesFormulario', compact('datosDelObjeto', 'routeName'))->render(),

            ];
        });
        return response()->json($almacenes);
    }



    public function cambiarEstatus(Request $request)
    {
        $idAlmacen = $request->input('idRegistroParaEditar');
        $almacen = Almacen::findOrFail($idAlmacen);
        $nuevoEstatus = $almacen->estatus === 'activo' ? 'inactivo' : 'activo';

        $almacen->estatus = $nuevoEstatus;
        $almacen->save();
        $estatusMensaje = $almacen->estatus === 'activo' ? 'Habilitado' : 'Inhabilitada';

        return response()->json(['mensaje' => 'Estatus del Almacen actualizado exitosamente.', 'nuevoEstatus' => $estatusMensaje]);
    }
}
