<?php

namespace App\Http\Controllers;

use App\Models\Categoria;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Validator;
use App\Models\Subcategoria;

class CategoriaController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        return view('categoria.index');
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     */

    public function store(Request $request)
    {
        // Validación
        $validator = Validator::make(
            $request->all(),
            [
                'nombre' => 'required|string|min:2|max:30|unique:categorias,nombre_categoria',
                'descripcion' => 'required|string|max:255',
            ],
            [
                'nombre.required' => 'El nombre de la categoría es obligatorio',
                'nombre.unique' => 'El nombre de la categoría ya está registrado',
                'nombre.min' => 'El nombre de la categoría debe tener al menos 2 caracteres',
                'nombre.max' => 'El nombre de la categoría no puede exceder los 30 caracteres',
                'descripcion.required' => 'La descripción es obligatoria',
            ]
        );

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()], 422);
        }

        try {
            // Crear la categoría
            $categoria = Categoria::create([
                'nombre_categoria' => $request->nombre,
                'descripcion' => $request->descripcion,
            ]);

            return response()->json(['mensaje' => 'Categoría creada exitosamente', 'categoria' => $categoria], 201);
        } catch (\Exception $e) {
            // Loguear el error para depuración
            Log::error('Error al registrar la categoría: ' . $e->getMessage());

            // Retornar respuesta de error genérico
            return response()->json(['mensaje' => 'Ocurrió un error al registrar la categoría.'], 500);
        }
    }

    /**
     * Display the specified resource.
     */
    public function show(Categoria $categoria)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(Request $request)
    {
        $idCategoria = $request->input('idRegistroParaEditar');
        $dataCategoria = Categoria::findOrFail($idCategoria);
        return response()->json($dataCategoria);
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, Categoria $categoria)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Categoria $categoria)
    {
        //
    }

    public function obtenerCategorias()
    {
        $routeName = 'categoria'; // Puedes modificar esto según el módulo
        $almacenes = Categoria::withCount('productos')->get()->map(function ($datosDelObjeto) use ($routeName) {
            return [
                'id'                => $datosDelObjeto->id,
                'nombre'           => $datosDelObjeto->nombre_categoria,
                'descripcion'         => $datosDelObjeto->descripcion,
                'estatus'          => $datosDelObjeto->estatus,
                'cantArticulos'         => $datosDelObjeto->productos_count,
                'acciones'          => view('components.accionesFormulario', compact('datosDelObjeto', 'routeName'))->render(),

            ];
        });
        return response()->json($almacenes);
    }

    public function getSubcategorias($id)
    {
        $routeName = 'subcategoria'; // Puedes modificar esto según el módulo
        $subcategorias = Categoria::find($id)->subcategorias->map(function ($datosDelObjeto) use ($routeName) {
            return [
                'id'                => $datosDelObjeto->id,
                'nombre'            => $datosDelObjeto->nombre,
                'descripcion'       => $datosDelObjeto->descripcion,
                'acciones'          => view('components.accionesFormulario', compact('datosDelObjeto', 'routeName'))->render(),
            ];
        });
        return response()->json($subcategorias);
    }



    public function cambiarEstatus(Request $request)
    {
        $idCategoria = $request->input('idRegistroParaEditar');
        $categoria = Categoria::findOrFail($idCategoria);
        $nuevoEstatus = $categoria->estatus === 'activo' ? 'inactivo' : 'activo';

        $categoria->estatus = $nuevoEstatus;
        $categoria->save();
        $estatusMensaje = $categoria->estatus === 'activo' ? 'Habilitado' : 'Inhabilitada';

        return response()->json(['mensaje' => 'Estatus de la Categoria actualizada exitosamente.', 'nuevoEstatus' => $estatusMensaje]);
    }

    public function storeSubCategoria(Request $request)
    {
        // Validación
        $validator = Validator::make(
            $request->all(),
            [
                'nombre' => [
                    'required',
                    'string',
                    'min:2',
                    'max:30',
                    function ($attribute, $value, $fail) use ($request) {
                        $exists = Subcategoria::where('nombre', $value)
                            ->where('categoria_id', $request->categoria_id)
                            ->exists();
                        if ($exists) {
                            $fail('El nombre de la subcategoría ya está registrado en esta categoría.');
                        }
                    },
                ],
                'descripcion' => 'required|string|max:255',
                'idCategoriaRegistroSubCategoria' => 'required|exists:categorias,id',
            ],
            [
                'nombre.required' => 'El nombre de la subcategoría es obligatorio',
                'nombre.min' => 'El nombre de la subcategoría debe tener al menos 2 caracteres',
                'nombre.max' => 'El nombre de la subcategoría no puede exceder los 30 caracteres',
                'descripcion.required' => 'La descripción es obligatoria',
                'idCategoriaRegistroSubCategoria.required' => 'La categoría es obligatoria',
                'idCategoriaRegistroSubCategoria.exists' => 'La categoría seleccionada no es válida',
            ]
        );

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()], 422);
        }

        try {
            // Crear la subcategoría
            $subcategoria = Categoria::find($request->idCategoriaRegistroSubCategoria)->subcategorias()->create([
                'nombre' => $request->nombre,
                'descripcion' => $request->descripcion,
            ]);

            return response()->json(['mensaje' => 'Subcategoría creada exitosamente', 'subcategoria' => $subcategoria], 201);
        } catch (\Exception $e) {
            // Loguear el error para depuración
            Log::error('Error al registrar la subcategoría: ' . $e->getMessage());

            // Retornar respuesta de error genérico
            return response()->json(['mensaje' => 'Ocurrió un error al registrar la subcategoría.'], 500);
        }
    }
}
