<?php

namespace App\Http\Controllers;

use App\Models\Sucursales;
use App\Http\Requests\StoreSucursalRequest;
use App\Http\Requests\UpdateSucursalRequest;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Log;

class SucursalController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {

        return view('sucursal.index');
    }





    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(StoreSucursalRequest $request)
    {
        // Validación
        $validator = Validator::make(
            $request->all(),
            [
                'nombre_sucursal' => 'required|string|max:50',
                'codigo_sucursal' => 'required|string|max:50|unique:sucursales,codigo_sucursal',
                'telefono' => 'required|string|max:15',
                'direccion' => 'required|string|max:255',
            ],
            [
                'nombre_sucursal.required' => 'El nombre de la sucursal es obligatorio',
                'codigo_sucursal.required' => 'El código de la sucursal es obligatorio',
                'codigo_sucursal.unique' => 'El código de la sucursal ya está registrado',
                'telefono.required' => 'El teléfono es obligatorio',
                'direccion.required' => 'La dirección es obligatoria',
            ]
        );

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()], 422);
        }

        try {
            // Crear la sucursal
            $sucursal = Sucursales::create([
                'nombre_sucursal' => $request->nombre_sucursal,
                'codigo_sucursal' => $request->codigo_sucursal,
                'telefono' => $request->telefono,
                'direccion' => $request->direccion,
                'estatus' => 1, // Asumiendo que estatus es un campo requerido
            ]);

            return response()->json(['mensaje' => 'Sucursal creada exitosamente', 'sucursal' => $sucursal], 201);
        } catch (\Exception $e) {
            // Loguear el error para depuración
            Log::error('Error al registrar la sucursal: ' . $e->getMessage());

            // Retornar respuesta de error genérico
            return response()->json(['mensaje' => 'Ocurrió un error al registrar la sucursal.'], 500);
        }
    }


    /**
     * Display the specified resource.
     */
    public function show(Sucursales $sucursal)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(Request $request)
    {
        $idSucursal = $request->input('idRegistroParaEditar');
        $sucursal = Sucursales::findOrFail($idSucursal);
        return response()->json($sucursal);
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(UpdateSucursalRequest $request)
    {


        $idSucursal = $request->input('idSucursalEdicion');

        // Validaciones
        $validatedData = $request->validate([
            'nombre_sucursal_edicion' => 'required|min:5|max:50',
            'telefono_edicion' => 'required|numeric|digits:11',
            'direccion_edicion' => 'required|min:5|max:200',
            'codigo_sucursal_edicion' => [
                'required',
                Rule::unique('sucursales', 'codigo_sucursal')->ignore($idSucursal)
            ],
        ], [
            'nombre_sucursal_edicion.required' => 'Campo Nombre de la Sucursal es Requerido',
            'nombre_sucursal_edicion.min' => 'El nombre de la Sucursal requiere 5 caracteres mínimo',
            'nombre_sucursal_edicion.max' => 'El nombre de la Sucursal no puede exceder 50 caracteres',

            'telefono_edicion.required' => 'Campo Teléfono Local es Requerido',
            'telefono_edicion.numeric' => 'El campo Teléfono debe ser numérico',
            'telefono_edicion.digits' => 'Campo Teléfono local requiere 11 caracteres',

            'direccion_edicion.required' => 'Campo Dirección es Requerido',
            'direccion_edicion.min' => 'Campo Dirección requiere 5 caracteres mínimo',
            'direccion_edicion.max' => 'Campo Dirección no puede exceder los 200 caracteres',

            'codigo_sucursal_edicion.required' => 'Campo Código es Requerido',
            'codigo_sucursal_edicion.unique' => 'El Código de la Sucursal debe ser único'
        ]);



        // Encontrar la sucursal y actualizar los datos
        $sucursal = Sucursales::findOrFail($idSucursal);
        $sucursal->nombre_sucursal = $request->input('nombre_sucursal_edicion');
        $sucursal->telefono = $request->input('telefono_edicion');
        $sucursal->direccion = $request->input('direccion_edicion');
        $sucursal->codigo_sucursal = $request->input('codigo_sucursal_edicion');
        $sucursal->save();

        return response()->json(['mensaje' => 'Sucursal actualizada exitosamente.']);
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Sucursales $sucursal)
    {
        //
    }

    private function validacionesYProcesos(Request $request)
    {
        $request->validate([
            'nombre_sucursal' => 'required|min:5|max:50',
            'telefono' => 'required|numeric|digits:11',
            'direccion' => 'required|min:5|max:200',
            'codigo_sucursal' => [
                'required',
                Rule::unique('sucursales', 'codigo_sucursal')
            ],
        ], [
            'nombre_sucursal.min' => 'El nombre de la Sucursal requiere 5 caracteres mínimo',
            'nombre_sucursal.max' => 'El nombre de la Sucursal no puede exceder 50 caracteres',
            'nombre_sucursal.required' => 'Campo Nombre de la Sucursal es Requerido',

            'telefono.required' => 'Campo Teléfono Local es Requerido',
            'telefono.numeric' => 'El campo Teléfono debe ser numérico',
            'telefono.digits' => 'Campo Teléfono local requiere 11 caracteres',

            'direccion.required' => 'Campo Dirección es Requerido',
            'direccion.min' => 'Campo Dirección requiere 5 caracteres mínimo',
            'direccion.max' => 'Campo Dirección no puede exceder los 200 caracteres',

            'codigo_sucursal.required' => 'Campo Código es Requerido',
            'codigo_sucursal.unique' => 'El Código de la Sucursal debe ser único'
        ]);

        $data = $request->only(['nombre_sucursal', 'codigo_sucursal', 'telefono', 'direccion']);

        return $data;
    }

    public function obtenerSucursales()
    {
        $routeName = 'sucursales'; // Puedes modificar esto según el módulo
        $sucursales = Sucursales::withCount('almacenes')->get()->map(function ($datosDelObjeto) use ($routeName) {
            return [
                'id' => $datosDelObjeto->id,
                'nombre_sucursal'   => $datosDelObjeto->nombre_sucursal,
                'descripcion'       => $datosDelObjeto->descripcion,
                'telefono'          => $datosDelObjeto->telefono,
                'direccion'         => $datosDelObjeto->direccion,
                'codigo_sucursal'   => $datosDelObjeto->codigo_sucursal,
                'estatus'           => $datosDelObjeto->estatus,
                'cantAlmacenes'     => $datosDelObjeto->almacenes_count,
                'acciones'          => view('components.accionesFormulario', compact('datosDelObjeto', 'routeName'))->render(),
            ];
        });
        return response()->json($sucursales);
    }



    public function cambiarEstatus(Request $request)
    {
        $idSucursal = $request->input('idRegistroParaEditar');
        $sucursal = Sucursales::findOrFail($idSucursal);
        $nuevoEstatus = $sucursal->estatus === 'activo' ? 'inactivo' : 'activo';

        $sucursal->estatus = $nuevoEstatus;
        $sucursal->save();
        $estatusMensaje = $sucursal->estatus === 'activo' ? 'Habilitada' : 'Inhabilitada';

        return response()->json(['mensaje' => 'Estatus de la sucursal actualizado exitosamente.', 'nuevoEstatus' => $estatusMensaje]);
    }

    public function obtenerSucursalesHabilitadas()
    {
        $sucursales = Sucursales::where('estatus', 'activo')
            ->withCount('almacenes')
            ->get();

        return response()->json($sucursales);
    }
}
